
SET NOCOUNT ON;
USE master;
GO

DROP DATABASE
     dbUDFTest;
GO

CREATE DATABASE dbUDFTest;
GO

USE dbUDFTest;
GO

CREATE FUNCTION dbo.fnSerialData(
                @Data smalldatetime )
RETURNS float
AS
     BEGIN
         RETURN CONVERT( float , YEAR( @Data )) * 10000 + CONVERT( float , MONTH( @Data )) * 100 + CONVERT( float , DAY( @Data ));
     END;
GO

CREATE FUNCTION dbo.fnConvertDateTimeToLinuxDATE(
                @dt datetime )
RETURNS char( 8 )
AS
     BEGIN
         DECLARE @ret char( 8 );
         DECLARE @month nvarchar( 2 );
         DECLARE @day nvarchar( 2 );

         SET @month = CONVERT( nvarchar( 2 ) , DATEPART( mm , @dt ));

         IF LEN( @month ) = 1
             BEGIN
                 SET @month = '0' + @month;
             END;

         SET @day = CONVERT( nvarchar( 2 ) , DATEPART( dd , @dt ));

         IF LEN( @day ) = 1
             BEGIN
                 SET @day = '0' + @day;
             END;
         SET @ret = CONVERT( char( 4 ) , DATEPART( yyyy , @dt )) + @month + @day;

         RETURN @ret;
     END;
GO

CREATE TABLE regioni(
             idRegione int , 
             descrizione varchar( 35 ));
GO

INSERT INTO regioni
VALUES
       ( 1 , 'liguria' ),
       ( 2 , 'lombardia' ),
       ( 3 , 'lazio' ),
       ( 4 , 'piemonte' ),
       ( 5 , 'marche' ),
       ( 6 , 'abruzzo' );
GO

CREATE FUNCTION dbo.fnTornaRegione(
                @i int )
RETURNS varchar( 35 )
AS
     BEGIN
         RETURN( SELECT
                        descrizione
                   FROM regioni
                   WHERE idRegione = @i );
     END;
GO

CREATE FUNCTION dbo.fnTornaCalcolo(
                @a int , 
                @b int )
RETURNS int
AS
     BEGIN
         RETURN @a + @b;
     END;
GO


CREATE TABLE dati(
             idRecord uniqueidentifier PRIMARY KEY , 
             data1 datetime , 
             data2 datetime , 
             data3 datetime , 
             data4 datetime , 
             data5 datetime , 
             testo varchar( 35 ) , 
             idRegione int , 
             valore1 int , 
             valore2 int );
GO



DECLARE @i int;
SET @i = 50000;
WHILE @i > 0
    BEGIN
        INSERT INTO dati
        VALUES
               ( NEWID( ) , GETDATE( ) , GETDATE( ) , GETDATE( ) , GETDATE( ) , GETDATE( ) , 'campo...' , 2 , 5 , 6 );
        SET @i = @i - 1;
    END;
GO



SELECT
       COUNT( * )
  FROM dati; 


select top 10 * from dati


/* PRIMA ANALISI */

SET STATISTICS IO ON;
SET STATISTICS TIME ON;
SELECT
       dbo.fnConvertDateTimeToLinuxDATE( data1 ) , 
       dbo.fnSerialData( data2 ) , 
       dbo.fnSerialData( data3 ) , 
       dbo.fnSerialData( data4 ) , 
       dbo.fnSerialData( data5 ) , 
       testo , 
       dbo.fnTornaRegione( idRegione )AS         regione , 
       dbo.fnTornaCalcolo( valore1 , valore2 )AS valore
  FROM dati;
SET STATISTICS TIME OFF;
SET STATISTICS IO OFF;
GO
--> CPU time = XXX ms,  elapsed time = XXX ms
--> Table 'dati'. Scan count XXX, logical reads XXX


/* SECONDA ANALISI */

SET STATISTICS IO ON;
SET STATISTICS TIME ON;
SELECT
       CONVERT( char( 8 ) , data1 , 112 ) , 
       CONVERT( char( 8 ) , data2 , 112 ) , 
       CONVERT( char( 8 ) , data3 , 112 ) , 
       CONVERT( char( 8 ) , data4 , 112 ) , 
       CONVERT( char( 8 ) , data5 , 112 ) , 
       testo , 
       regioni.descrizione AS regione , 
       valore1 + valore2 AS   valore
  FROM dati
       JOIN regioni ON dati.idregione = regioni.idRegione;
SET STATISTICS TIME OFF;
SET STATISTICS IO OFF;
GO
--> CPU time = XXX ms,  elapsed time = XXX ms
--> Table 'dati'. Scan count XXX, logical reads XXX


