
/* NOLOCK 2 */


--> setup
USE AdventureWorks2012;
GO
SET NOCOUNT ON;

IF OBJECT_ID('dbo.ProductDealerInformationByOrder','U') IS NOT NULL
    DROP TABLE dbo.ProductDealerInformationByOrder;
GO

CREATE TABLE dbo.ProductDealerInformationByOrder 
    (ProductId INT,
     OrderDate DATETIME,
     DealerListPrice AS (dbo.[ufnGetProductDealerPrice](ProductId, OrderDate))
    );
GO

--Populate some test data
INSERT INTO dbo.ProductDealerInformationByOrder (ProductId, OrderDate)
SELECT sod.ProductID,
       soh.OrderDate
FROM Sales.SalesOrderHeader AS soh
INNER JOIN Sales.SalesOrderDetail AS sod ON soh.SalesOrderID = sod.SalesOrderID
WHERE soh.SalesOrderID = 51721;
GO


--> update table
USE AdventureWorks2012;
GO
-- Note the session number!
BEGIN TRANSACTION LockProductListPriceHistory
    UPDATE plph
    SET plph.ListPrice = (plph.ListPrice + 10)
    FROM Sales.SalesOrderHeader AS soh
    INNER JOIN Sales.SalesOrderDetail AS sod ON soh.SalesOrderID = sod.SalesOrderID
    INNER JOIN Production.ProductListPriceHistory AS plph ON sod.ProductID = plph.ProductID
    WHERE soh.SalesOrderID = 51721;

    --Check the nature of the locks applied
    SELECT sdtl.request_type AS LockRequeust,
           sdtl.request_status AS LockRequestStatus,
           sdtl.request_mode AS LockMode,
           sdtl.resource_type AS ObjectType,
           DB_NAME(sdtl.resource_database_id) AS DatabaseName,
           SCHEMA_NAME(so.schema_id) AS SchemaName,
           so.name AS ObjectName,
           sdtl.request_session_id AS SessionNumber
    FROM sys.dm_tran_locks AS sdtl
    INNER JOIN sys.objects AS so ON sdtl.resource_associated_entity_id = so.object_id;
--ROLLBACK TRANSACTION LockProductListPriceHistory


--> select (new query)
/* Run this in a new SSMS query editor window. */
USE AdventureWorks2012;
GO
BEGIN TRANSACTION SelectWithNoLock
    ;WITH ProductInfoCTE (ProductId, OrderDate, DealerListPrice)
    AS (SELECT pdio.ProductId,
               pdio.OrderDate,
               pdio.DealerListPrice
        FROM dbo.ProductDealerInformationByOrder AS pdio WITH (NOLOCK)
       )
    SELECT DISTINCT sdtl.request_type AS LockRequest,
                    sdtl.request_status AS LockRequestStatus,
                    sdtl.request_mode AS LockMode,
                    sdtl.resource_type AS ObjecType,
                    DB_NAME(sdtl.resource_database_id) AS DatabaseName,
                    SCHEMA_NAME(so.schema_id) AS SchemaName,
                    so.name AS ObjectName,
                    sdtl.request_session_id AS SessionNumber,
                    pcte.ProductId,
                    pcte.OrderDate,
                    pcte.DealerListPrice
    FROM sys.dm_tran_locks AS sdtl
    INNER JOIN sys.objects AS so ON sdtl.resource_associated_entity_id = so.object_id
    CROSS JOIN ProductInfoCTE AS pcte;
ROLLBACK TRANSACTION SelectWithNoLock
GO


--> examine locks (new query)
/* Run this in a new SSMS query editor window (window #3). */
USE AdventureWorks2012;
GO
SELECT sdtl.request_type AS LockRequeust,
        sdtl.request_status AS LockRequestStatus,
        sdtl.request_mode AS LockMode,
        sdtl.resource_type AS ObjectType,
        DB_NAME(sdtl.resource_database_id) AS DatabaseName,
        SCHEMA_NAME(so.schema_id) AS SchemaName,
        so.name AS ObjectName,
        sdtl.request_session_id AS SessionNumber,
        sr.blocking_session_id AS BlockingSession
FROM sys.dm_tran_locks AS sdtl
INNER JOIN sys.objects AS so ON sdtl.resource_associated_entity_id = so.object_id
LEFT OUTER JOIN sys.dm_exec_requests AS sr ON sdtl.request_session_id = sr.session_id
ORDER BY sr.session_id;
GO